@@ -4,7 +4,7 @@ by a chooser for LSR.
 
 @author: Peter Parente
 @organization: IBM Corporation
-@copyright: Copyright (c) 2005 IBM Corporation
+@copyright: Copyright (c) 2005, 2006 IBM Corporation
 @license: Common Public License 1.0
 
 All rights reserved. This program and the accompanying materials are made 
@@ -12,6 +12,4 @@ available under the terms of the Common Public License v1.0 which accompanies
 this distribution, and is available at
 U{http://www.opensource.org/licenses/cpl1.0.php}
 '''
-
-def isAEChooser(obj):
-  return True
\ No newline at end of file
+from Base import *
