@@ -186,7 +186,7 @@ class ContainerAccInfoAdapter(DefaultAccInfoAdapter):
       c = self.accessible.getChildAtIndex(self.item_offset)
     except AttributeError:
       raise IndexError
-    return IAccessibleInfo(POR(c, self.item_offset)).getAccStates()
+    return IAccessibleInfo(POR(c)).getAccStates()
   
   @pyLinAcc.errorToLookupError 
   def getAccIndex(self):
@@ -197,7 +197,10 @@ class ContainerAccInfoAdapter(DefaultAccInfoAdapter):
     @rtype: integer
     @raise LookupError: When the table or item is no longer valid
     '''
-    return self.item_offset
+    if self.item_offset is None:
+      return super(ContainerAccInfoAdapter, self).getAccIndex()
+    else:
+      return self.item_offset
   
   @pyLinAcc.errorToLookupError
   def hasAccState(self, state):
@@ -216,7 +219,7 @@ class ContainerAccInfoAdapter(DefaultAccInfoAdapter):
       c = self.accessible.getChildAtIndex(self.item_offset)
     except AttributeError:
       raise IndexError
-    return IAccessibleInfo(POR(c, self.item_offset)).hasAccState(state)
+    return IAccessibleInfo(POR(c)).hasAccState(state)
   
   @pyLinAcc.errorToLookupError
   def hasAccOneState(self, *states):
@@ -234,7 +237,7 @@ class ContainerAccInfoAdapter(DefaultAccInfoAdapter):
       c = self.accessible.getChildAtIndex(self.item_offset)
     except AttributeError:
       raise IndexError
-    return IAccessibleInfo(POR(c, self.item_offset)).hasAccOneState(*states)
+    return IAccessibleInfo(POR(c)).hasAccOneState(*states)
   
   @pyLinAcc.errorToLookupError
   def hasAccAllStates(self, *states):
@@ -252,7 +255,7 @@ class ContainerAccInfoAdapter(DefaultAccInfoAdapter):
       c = self.accessible.getChildAtIndex(self.item_offset)
     except AttributeError:
       raise IndexError
-    return IAccessibleInfo(POR(c, self.item_offset)).hasAccAllStates(*state)
+    return IAccessibleInfo(POR(c)).hasAccAllStates(*state)
   
   @pyLinAcc.errorToLookupError
   def getAccActionNames(self):
@@ -269,7 +272,7 @@ class ContainerAccInfoAdapter(DefaultAccInfoAdapter):
       c = self.accessible.getChildAtIndex(self.item_offset)
     except AttributeError:
       raise IndexError
-    return IAccessibleInfo(POR(c, self.item_offset)).getAccActionNames()
+    return IAccessibleInfo(POR(c)).getAccActionNames()
   
   @pyLinAcc.errorToLookupError
   def getAccActionDescs(self):
@@ -286,7 +289,7 @@ class ContainerAccInfoAdapter(DefaultAccInfoAdapter):
       c = self.accessible.getChildAtIndex(self.item_offset)
     except AttributeError:
       raise IndexError
-    return IAccessibleInfo(POR(c, self.item_offset)).getAccActionDescs()
+    return IAccessibleInfo(POR(c)).getAccActionDescs()
 
   @pyLinAcc.errorToLookupError
   def getAccActionCount(self):
@@ -303,7 +306,7 @@ class ContainerAccInfoAdapter(DefaultAccInfoAdapter):
       c = self.accessible.getChildAtIndex(self.item_offset)
     except AttributeError:
       raise IndexError
-    return IAccessibleInfo(POR(c, self.item_offset)).getAccActionCount()
+    return IAccessibleInfo(POR(c)).getAccActionCount()
 
   @pyLinAcc.errorToLookupError
   def getAccActionKeys(self):
@@ -326,7 +329,42 @@ class ContainerAccInfoAdapter(DefaultAccInfoAdapter):
       c = self.accessible.getChildAtIndex(self.item_offset)
     except AttributeError:
       raise IndexError
-    return IAccessibleInfo(POR(c, self.item_offset)).getAccActionKeys()
+    return IAccessibleInfo(POR(c)).getAccActionKeys()
+
+  @pyLinAcc.errorToLookupError
+  def getAccVisualExtents(self):
+    '''
+    Gets the visual width and height of the subject.
+
+    @return: Width and height extents
+    @rtype: 2-tuple of integer
+    @raise LookupError: When the accessible object is dead
+    '''
+    if self.item_offset is None:
+      return super(ContainerAccInfoAdapter, self).getAccVisualExtents()
+    try:
+      c = self.accessible.getChildAtIndex(self.item_offset)
+    except AttributeError:
+      raise IndexError
+    return IAccessibleInfo(POR(c)).getAccVisualExtents()
+
+  @pyLinAcc.errorToLookupError
+  def getAccVisualPoint(self):
+    '''
+    Gets the focal point within the subject, where the focal point is defined
+    as the center of the active item.
+
+    @return: x,y coordinates
+    @rtype: 2-tuple of integer
+    @raise LookupError: When the accessible object is dead
+    '''
+    if self.item_offset is None:
+      return super(ContainerAccInfoAdapter, self).getAccVisualPoint()
+    try:
+      c = self.accessible.getChildAtIndex(self.item_offset)
+    except AttributeError:
+      raise IndexError
+    return IAccessibleInfo(POR(c)).getAccVisualPoint()
   
 class ContainerAccActionAdapter(DefaultAccActionAdapter):
   '''
@@ -411,4 +449,4 @@ class ContainerAccActionAdapter(DefaultAccActionAdapter):
     else:
       child = self.accessible.getChildAtIndex(self.item_offset)
       act = Interfaces.IAction(child)
-      return act.doAction(index)
\ No newline at end of file
+      return act.doAction(index)
