@@ -90,6 +90,16 @@ class POR(object):
              (self.accessible == other.accessible)
     except:
       return False
+    
+  def __hash__(self):
+    '''
+    Hashes based on the accessible, item offset, and character offset rather 
+    than the ID of the L{POR} itself.
+    
+    @return: Immutable hash code based on the attributes of the L{POR}
+    @rtype: integer
+    '''
+    return hash(self.accessible)^(self.item_offset or 0)^(self.char_offset)
   
   def __ne__(self, other):
     '''
@@ -193,4 +203,4 @@ class POR(object):
     @return: Item after the other?
     @rtype: boolean
     '''
-    return self.item_offset > other.item_offset
\ No newline at end of file
+    return self.item_offset > other.item_offset
