@@ -1,23 +1,6 @@
 '''
 Defines the default user interface for LSR.
 
-Event announcements
-
-(X) means optional announcement X based on type of widget
-[X] means X always announced
-<X> indicates continue announcements in X
-
-on view change: [window title]
-on focus: (container) (new widget role) <active descendant change>
-on active descendant change: (new item role) (new level) [item] [state] 
-  [hotkey] (value) (value extents)
-on caret move: [characters passed]
-on text insert: [characters inserted]
-on text delete: [characters deleted]
-on state changed: [new state]
-on alert: [all labels in dialog] <focus>
-on value change: [new value]
-
 @author: Peter Parente
 @author: Pete Brunet
 @author: Larry Weiss
@@ -544,90 +527,6 @@ class ReadMessage(Task.Task):
     '''
     self.say(text, sem=sem)
 
-class AnnounceUIERaise(Task.Task):   
-  '''
-  Read the user interface element above and below the one that was just raised
-  to a higher priority.
-  
-  @deprecated
-  '''
-  def execute(self, name, above, below, **kwargs):
-    '''
-    Announces that a L{UIElement} moved up in the load order.
-    
-    @param name: Name of the L{UIElement}
-    @type name: string
-    @param above: Name of the L{UIElement} above
-    @type above: string
-    @param below: Name of the L{UIElement} below
-    @type below: string
-    '''
-    self.sayItem(text=name, template=_('raised %s above %s, below %s') % 
-                 ('%s', below, above))
-  
-class AnnounceUIELower(Task.Task):
-  '''  
-  Read the user interface element above and below the one that was just
-  lowered to a lower priority. 
-  
-  @deprecated
-  '''
-  def execute(self, name, above, below, **kwargs):
-    '''
-    Announces that a L{UIElement} moved down in the load order.
-    
-    @param name: Name of the L{UIElement}
-    @type name: string
-    @param above: Name of the L{UIElement} above
-    @type above: string
-    @param below: Name of the L{UIElement} below
-    @type below: string
-    '''
-    self.sayItem(text=name, template=_('lowered %s below %s, above %s') % 
-                 ('%s', above, below))
-    
-class AnnouncePerkLoad(Task.Task):
-  '''
-  Announce that a Perk is now configured to be loaded on the current Tier.
-  
-  @deprecated
-  '''
-  def execute(self, name, **kwargs):
-    '''
-    Announces that a L{Perk} is now in the list of L{Perk}s to load.
-    
-    @param name: Name of the L{Perk}
-    @type name: string
-    '''
-    str = _('added %s to top of %s') % (name, self.getAppName())
-    self.sayItem(text=str)
-  
-class AnnouncePerkUnload(Task.Task):
-  '''
-  Announce that a Perk is now configured to be unloaded from the current
-  Tier.
-  
-  @deprecated
-  '''
-  def execute(self, name, **kwargs):
-    '''
-    Announces that a L{Perk} is now in the list of L{Perk}s to unload.
-    
-    @param name: Name of the L{Perk}
-    @type name: string
-    '''
-    str = _('removed %s from %s') % (name, self.getAppName())
-    self.sayItem(text=str)
-    
-class AnnounceConfirmation(Task.Task):
-  '''
-  Announce an arbitrary confirmation on an output device.
-  
-  @deprecated
-  '''
-  def execute(self, text, **kwargs):
-    self.sayConfirm(text=text)
-
 class ReadAll(Task.InputTask):
   '''
   Read the remaining contents of the active view starting at the current
