@@ -1,5 +1,22 @@
 '''
 Defines a user interface for gaim.
+
+Keys registered
+
+CapsLock-W: Reports status of all open conversations
+CapsLock-S: Cycle focus between message history and compose area
+
+Event announcements
+
+(X) means optional announcement X based on type of widget
+[X] means X always announced
+<X> indicates continue announcements in X
+
+on activate: ["chat with" username]
+on focus: ["compose" or "history"] <active descendant change>
+on message received: [text of message]
+
+@todo: PP: redocument before 0.3 release
   
 @var CONTAINER_PATH: path from frame to container with accessible elements
 @type CONTAINER_PATH: integer
@@ -53,10 +70,12 @@ available under the terms of the Common Public License v1.0 which accompanies
 this distribution, and is available at
 U{http://www.opensource.org/licenses/cpl1.0.php}
 '''
-import Perk, Task
+import Perk, Task, AEState
 from POR import POR
 from i18n import _
 
+__uie__ = dict(kind='perk', tier='gaim')
+
 # actual paths, derived using at-poke, assumed to be static.  YMMV.
 # path: from frame to container with accessible elements   
 CONTAINER_PATH = 0
@@ -89,18 +108,35 @@ LOGOUT_STATUS = 6
 STATUS_MSG = [_('unseen'), _('typing'), _('paused'), _('idle'), _('offline'),
               _('logged in'), _('logged out')] 
 
+class GaimPerkState(Perk.PerkState):
+  '''
+  Settings for gaim.
+  
+  @todo: PP: mark _ after 0.3.0
+  
+  @cvar ReadBackground: Read incoming messages when gaim is in the background?
+  @type ReadBackground: boolean
+  '''
+  ReadBackground = False
+  
+  def getSettings(self):
+    g = self._newGroup()
+    g.newBool('ReadBackground', 'Read background messages?', 'When checked, '
+              'incoming messages will be read even when gaim is in the '
+              'background.')
+    return g
+
 class GaimPerk(Perk.Perk):
   '''
-  Supplements the DefaultPerk to provide better access to the gaim instant
-  messaging client. Defines keyboard commands for querying conversation status
-  and cycling focus between the message history and composition text areas. 
-  Announces incoming messages and allows for the review of entire messages (not
-  lines of text) in the history. Has initial support for announcing when
-  buddies sign in and out and when the status of a conversation changes. Written
-  to work with both gaim 1.5 and 2.0.
+  Provides hotkeys for switching between the message history and message 
+  compose areas. Announces incoming messages and allows for review by entire
+  message. Supplements the DefaultPerk.
   
-  The following instance variables are attached to the L{perk_state} object
-  and shared across all L{Task}s in this L{Perk}.
+  Has initial support for announcing when buddies sign in and out and when the
+  status of a conversation changes. Written to work with both gaim 1.5 and 2.0.
+  
+  @todo: DBC: read BuddyList, get ToolTip info per "buddy"
+  @todo: DBC: handle multiple chat windows
   
   @ivar chat_compose_por: Reference to chat msg compose panel
   @type chat_compose_por: L{POR}
@@ -116,89 +152,76 @@ class GaimPerk(Perk.Perk):
   @type is_history_focused: boolean
   @ivar last_history_caret: track caret in history panel, read when on new line
   @type last_history_caret: L{POR}
-
-  @todo: DBC: read BuddyList, get ToolTip info per "buddy"
-  @todo: DBC: handle multiple chat windows
-  '''  
+  '''
+  STATE = GaimPerkState
+  
   def init(self):
-    '''   
-    Registers keyboard bindings:   
-    
-    ALT(R)-SHIFT(R)-W: Gives status of conversation tabs: unread message,
-    active typing, paused typing
-    ALT(R)-SHIFT(R)-S: Drives focus back and forth between chat composition
-    text area and history text area.
-    
-    @todo: DBC: Maintain std nav keys but w/new modifiers for an app perk.
     '''
-    # create "global" Perk varibles to be referenced (by other classes belonging
-    # to this Perk) with "self.perk" (see below)
-    # Note: ".perk" will tell LSR where to find the variable that is
-    # "global" to this Perk.  While it's true that Python would have allowed
-    # a global variable to be placed within the file, there are good reasons
-    # (really!) why this approach is used in LSR (eg. inter-Perk communication)
-    # POR reference to chat composition text area
-    self.perk_state.chat_compose_por = POR() 
-    # POR reference to chat history text area
-    self.perk_state.chat_history_por = POR() 
-    self.perk_state.last_convo = POR()
-    
-    # have gaim environment variables been initialized?  This must happen first
-    # time that gaim chat window is activated
-    self.perk_state.initialized = False 
-    self.perk_state.is_gaim2 = False # is user client Gaim2 ?
-    
-    # does chat history panel currently have focus
-    self.perk_state.is_history_focused = False 
-    self.perk_state.last_history_caret = POR()
-    
-    # is there selection in the composition area? used to avoid announcing
-    # deleted text when sending
-    self.perk_state.selection = False
-
-    # may be used in case where, on sign-off, multiple inserts fire
-    #debug, see handleRowInserted, handleCaretChange
-    #last_row_inserted_por = None 
-
+    Registers keyboard bindings that give status of conversation tabs (unread
+    message, active typing, paused typing) and drive focus back and forth
+    between chat composition text area and history text area.
+    '''
     # register event handlers
     # events here are app-level, DefaultPerk will handle gnome-level events,
     # so only register events of interest here. 
-    self.registerEventTask(HandleViewChange(self))
-    self.registerEventTask(HandleFocusChange(self))
-    self.registerEventTask(HandleSelectorChange(self))
-    self.registerEventTask(HandleCaretChange(self))
-    self.registerEventTask(HandleTierCaretChange(self), focus=True, tier=True)
+    self.registerTask(ActivateGaimWindow())
+    self.registerTask(HandleFocusChange())
+    self.registerTask(HandleSelectorChange())
+    self.registerTask(HandleCaretChange())
+    self.registerTask(ReadIncoming(all=True))
     # not working for background events ???
-    #self.registerEventTask(HandlePropertyChange(self), focus=True, tier=True,
-    #                       background=True)
+    #self.registerEventTask(HandlePropertyChange(focus=True, tier=True,
+    #                       background=True))
     # @note: not handling TableChange until certain Gaim bugs fixed
     # if re-enabled, may want to uncomment debug code: last_por_inserted (global
     # variable) and code in handleCaretChange
-    #self.registerEventTask(HandleTableChange(self), focus=True, tier=True,
-                           #background=True)
+    #self.registerEventTask(BuddySignOnOff(focus=True, tier=True,
+                           #background=True))
     # see DefaultPerk.py for more event handling examples
     
     # register named tasks.  Specify the task class name (see below) and a 
     # description.  These descriptions are dictionary keys and therefore
     # must be unique.  Here, Gaim is a prefix to help ensure uniqueness.
-    self.registerNamedTask(ToggleChatPanel(self), 'Gaim chat panel toggle')
-    self.registerNamedTask(ConversationStatus(self), 'Gaim conversation status')
+    self.registerTask(ToggleChatPanel('gaim chat panel toggle'))
+    self.registerTask(ConversationStatus('gaim conversation status'))
 
     # get the Keyboard device and register modifiers.
     kbd = self.getInputDevice('Keyboard')
-    kbd.addModifier(kbd.AEK_CAPS_LOCK) # right alt key
+    self.addInputModifiers(kbd, kbd.AEK_CAPS_LOCK)
     
     # register the keys and their associated commands, binding them using the
     # unique dictionary key we registered above.
     # Pass 3 element tuple.  Need to capture keys as pressed though we only
     # handle them if all both modifiers pressed before the non-modifier.
-    self.registerCommandTask(kbd, [[kbd.AEK_CAPS_LOCK,
-                                    kbd.AEK_S]], 
-                             'Gaim chat panel toggle')
-    self.registerCommandTask(kbd, [[kbd.AEK_CAPS_LOCK,
-                                        kbd.AEK_W]], 
-                             'Gaim conversation status')
+    self.registerCommand(kbd, 'gaim chat panel toggle', False,
+                         [kbd.AEK_CAPS_LOCK, kbd.AEK_S])
+    self.registerCommand(kbd, 'gaim conversation status', False,
+                         [kbd.AEK_CAPS_LOCK, kbd.AEK_W])
     
+    # POR reference to chat composition text area
+    self.chat_compose_por = POR() 
+    # POR reference to chat history text area
+    self.chat_history_por = POR() 
+    self.last_convo = POR()
+  
+    # have gaim environment variables been initialized?  This must happen first
+    # time that gaim chat window is activated
+    self.initialized = False 
+    # is user client Gaim 2.0 instead of 1.5?
+    self.is_gaim2 = False 
+  
+    # does chat history panel currently have focus
+    self.is_history_focused = False 
+    self.last_history_caret = POR()
+  
+    # is there selection in the composition area? used to avoid announcing
+    # deleted text when sending
+    self.selection = False
+  
+    # may be used in case where, on sign-off, multiple inserts fire
+    #debug, see handleRowInserted, handleCaretChange
+    #last_row_inserted_por = None 
+
   def getActivePageTabListPOR(self):
     '''
     Gets a Point of Regard to the page tab list in an active chat window.
@@ -207,21 +230,17 @@ class GaimPerk(Perk.Perk):
     or None otherwise
     @rtype: L{POR}
     '''
-    try:
-      frame_por = self.getRootAcc()
-      container_por = self.getAccFromPath(frame_por, CONTAINER_PATH)
-      # is chat window active?
-      if (self.hasAccState('active', frame_por) and 
-          self.getAccCount(container_por) == NUM_CHILDREN_IN_CHAT_WINDOW):
-        # get 'page tab list'
-        tab_list_por = self.getAccFromPath(container_por, TAB_PATH)
-        if self.hasAccRole("page tab list", tab_list_por):
-          return tab_list_por
-      # in all other cases, return None
-      return None
-    # too general ???
-    except:
-      return None
+    frame_por = self.getRootAcc()
+    container_por = self.getAccFromPath(frame_por, CONTAINER_PATH)
+    # is chat window active?
+    if (self.hasAccState('active', frame_por) and 
+        self.getAccCount(container_por) == NUM_CHILDREN_IN_CHAT_WINDOW):
+      # get 'page tab list'
+      tab_list_por = self.getAccFromPath(container_por, TAB_PATH)
+      if self.hasAccRole("page tab list", tab_list_por):
+        return tab_list_por
+    # in all other cases, return None
+    return None
     
   def getTypingStatus(self, por):
     '''
@@ -257,10 +276,10 @@ class GaimPerk(Perk.Perk):
     tab_list_por = self.getActivePageTabListPOR()
     # get selected conversation tab
     selected_tab_por = self.getFirstSelected(tab_list_por)  
-    self.perk_state.chat_history_por = \
+    self.chat_history_por = \
         self.getAccFromPath(selected_tab_por, *HISTORY_PATH)
     # if this is a "filler" rather than a text area, must be Gaim2
-    if self.getAccRoleName(self.perk_state.chat_history_por) == "filler":
+    if self.getAccRoleName(self.chat_history_por) == "filler":
       self.is_gaim2 = True
       print "chat client is Gaim beta 2 or higher"
     else:
@@ -283,17 +302,19 @@ class ToggleChatPanel(Task.InputTask):
     # Note: ".perk" lets LSR find cross-class variables "global" to the Perk
     tab_list_por = self.perk.getActivePageTabListPOR()
     if tab_list_por is not None:
-      if self.hasAccState('focused', self.perk_state.chat_compose_por):
+      if self.hasAccState('focused', self.perk.chat_compose_por):
         #change focus to chat history panel
-        self.setFocus(self.perk_state.chat_history_por)  
+        self.setAccFocus(self.perk.chat_history_por)  
       else:
         #change focus to chat compose panel
-        self.setFocus(self.perk_state.chat_compose_por) 
+        self.setAccFocus(self.perk.chat_compose_por) 
     return True # allow triggering keystrokes to propagate (default)
 
 class ConversationStatus(Task.InputTask):
   '''
-  @todo: DBC: comment
+  Announces the status of the current conversations. Status for conversations
+  is grouped by status type except for the foreground conversation which is
+  always reported first.
   '''
   def execute(self, **kwargs):
     tab_list_por = self.perk.getActivePageTabListPOR()
@@ -320,7 +341,7 @@ class ConversationStatus(Task.InputTask):
                         template='%s %s,')
         else:
           l.append((value, name))
-      por = self.getNextAcc(por)
+      por = self.getNextPeerAcc(por)
         
     # sort the list of status values and names by priority, highest first)
     l.sort()
@@ -338,7 +359,7 @@ class ConversationStatus(Task.InputTask):
 # Tasks executed by events
 #
 
-class HandleViewChange(Task.ViewTask):
+class ActivateGaimWindow(Task.ViewTask):
   '''
   Task that handles a L{AEEvent.ViewChange}.
   '''
@@ -346,23 +367,24 @@ class HandleViewChange(Task.ViewTask):
     '''
     Prevents the window title from being announced if it is the chat window, 
     since better information is given for the chat window on the subsequent
-    focusChange event.  Accomplished by stopping propagation of the event.
+    focus change event. Accomplished by stopping propagation of the event.
     
     @param por: Point of regard to the root of the new view
     @type por: L{POR}
     @param title: Title of the newly activated window
     @type title: string
     '''
-    self.perk_state.last_convo = POR()
+    self.perk.last_convo = POR()
     tab_list_por = self.perk.getActivePageTabListPOR()
+    print 'GAIM:', tab_list_por
     # test to see if not a chat window
     if tab_list_por is None:
       return True # allow DefaultPerk to handle the view change
     # must be a chat window
     else:
-      if not self.perk_state.initialized:
+      if not self.perk.initialized:
         self.perk.setGaimVersion()
-        self.perk_state.initialized = True
+        self.perk.initialized = True
       # don't allow the view change event to propagate (ie. to DefaultPerk, 
       # where the frame title would be announced)
       return False
@@ -391,44 +413,42 @@ class HandleFocusChange(Task.FocusTask):
     selected_tab_por = self.getFirstSelected(tab_list_por)
     
     # check if we should announce the state of the current convo or not
-    if selected_tab_por != self.perk_state.last_convo:
+    if selected_tab_por != self.perk.last_convo:
       name = self.getAccName(selected_tab_por)
-      self.sayTitle(text=name, template=_('Chat with %s'))
+      self.saySection(text=name, template=_('chat with %s'))
       value = self.perk.getTypingStatus(selected_tab_por)
       self.sayState(text=STATUS_MSG[value])
-      self.perk_state.last_convo = selected_tab_por
+      self.perk.last_convo = selected_tab_por
 
     # locate message history/compose text box PORs   
     # gaim 2 local client
-    if self.perk_state.is_gaim2: 
-      self.perk_state.chat_history_por = \
+    if self.perk.is_gaim2: 
+      self.perk.chat_history_por = \
         self.getAccFromPath(selected_tab_por, *self.GAIM2_HISTORY_PATH)
       try:
-        self.perk_state.chat_compose_por = \
+        self.perk.chat_compose_por = \
           self.getAccFromPath(selected_tab_por, *self.GAIM2_COMPOSE_PATH)
       except:
-        self.perk_state.chat_compose_por = \
+        self.perk.chat_compose_por = \
           self.getAccFromPath(selected_tab_por, *self.GAIM2_ALT_COMPOSE_PATH)      
     # gaim 1.5 local client
     else:
-      self.perk_state.chat_history_por = \
+      self.perk.chat_history_por = \
           self.getAccFromPath(selected_tab_por, *HISTORY_PATH)
-      self.perk_state.chat_compose_por = \
+      self.perk.chat_compose_por = \
           self.getAccFromPath(selected_tab_por, *COMPOSE_PATH)   
-   
 
-    # tell user which panel is focused
-    
-    self.perk_state.is_history_focused = \
-        self.hasAccState('focused', self.perk_state.chat_history_por)
-    if self.perk_state.is_history_focused:
+    # tell user which text area is focused
+    self.perk.is_history_focused = \
+        self.hasAccState('focused', self.perk.chat_history_por)
+    if self.perk.is_history_focused:
     #if self.hasAccState('focused', self.perk.chat_history_por): 
-      self.sayTitle(text=_('History,')) # this odd syntax supports i18n
-      self.perk_state.last_history_caret = POR() # reset
+      self.saySection(text=_('history,')) # this odd syntax supports i18n
+      self.perk.last_history_caret = POR() # reset
       return False
-    elif self.hasAccState('focused', self.perk_state.chat_compose_por):
+    elif self.hasAccState('focused', self.perk.chat_compose_por):
     #elif self.hasAccState('focused', self.perk.chat_compose_por):
-      self.sayTitle(text=_('Compose,')) # this odd syntax supports i18n
+      self.saySection(text=_('compose,')) # this odd syntax supports i18n
       return False
     #else:
     #  return True # propagate handling of the event (default)
@@ -450,17 +470,16 @@ class HandleSelectorChange(Task.SelectorTask):
     @type text: string
     '''
     if text:
-      self.perk_state.selection = True
+      self.perk.selection = True
     else:
-      self.perk_state.selection = False
+      self.perk.selection = False
     return True
     
-class HandleTierCaretChange(Task.CaretTask):
+class ReadIncoming(Task.CaretTask):
   '''  
-  Task that handles a L{AEEvent.CaretChange} outside the focused control. Used 
-  to determine when a new message arrives and to speak it.
+  Reads incoming messages.
   '''
-  def executeInserted(self, por, text, text_offset, **kwargs):
+  def executeInserted(self, por, text, text_offset, layer, **kwargs):
     '''
     Speaks text that starts with a new line character. This indicates a new
     line of text has been inserted into the message history.
@@ -475,9 +494,14 @@ class HandleTierCaretChange(Task.CaretTask):
     @type text: string
     @param text_offset: The offset of the new caret position
     @type text_offset: integer
+    @param layer: Layer of the event, focus, tier, or background
+    @type layer: integer
     '''
+    if not self.perk_state.ReadBackground and layer == Task.LAYER_BACKGROUND:
+      # don't read background messages if the user has it disabled
+      return
     if text.startswith('\n'):
-      self.stopNow()
+      # queue up text to speak
       self.sayItem(text=text)
       
 class HandleCaretChange(Task.CaretTask):
@@ -510,16 +534,16 @@ class HandleCaretChange(Task.CaretTask):
     # could eliminate by setting is_history_focused = False when appropriate
     tab_list_por = self.perk.getActivePageTabListPOR()
     # if chat window and chat history have focus
-    if tab_list_por is not None and self.perk_state.is_history_focused:
+    if tab_list_por is not None and self.perk.is_history_focused:
       # say line when item_offset changes
-      if (not por.isSameItem(self.perk_state.last_history_caret)):
+      if (not por.isSameItem(self.perk.last_history_caret)):
         # get the start of the line (character after a \n)
         start = self.getStartOfHardLine(por)
         # get the end of the line (next encountered \n)
         end = self.getEndOfHardLine(por)
 
         # store the current POR
-        self.perk_state.last_history_caret = por
+        self.perk.last_history_caret = por
         # if we're not at the start or end of a true message line, let 
         # DefaultPerk handles the caret event unless at start/end of msg
         if por != end and por != start:
@@ -536,7 +560,7 @@ class HandleCaretChange(Task.CaretTask):
       # respond to caret movement on same line
       # just let DefaultPerk handle this case
       else: 
-        self.perk_state.last_history_caret = por
+        self.perk.last_history_caret = por
         return True
        
     # not in chat history
@@ -545,7 +569,7 @@ class HandleCaretChange(Task.CaretTask):
     
   def executeInserted(self, **kwargs):
     '''Resets the selection flag.'''
-    self.perk_state.selection = False
+    self.perk.selection = False
     return True
     
   def executeDeleted(self, text, **kwargs):
@@ -558,9 +582,9 @@ class HandleCaretChange(Task.CaretTask):
     @param text: The text deleted
     @type text: string
     '''
-    if not self.perk_state.selection and len(text) > 1:
+    if not self.perk.selection and len(text) > 1:
       return False
-    self.perk_state.selection = False
+    self.perk.selection = False
     return True
   
 class HandlePropertyChange(Task.PropertyTask):
@@ -589,9 +613,10 @@ class HandlePropertyChange(Task.PropertyTask):
     return True
   
  
-class HandleTableChange(Task.TableTask):
+class BuddySignOnOff(Task.TableTask):
   '''  
   Task that handles a L{AEEvent.TableChange}.
+  
   These events will be used to know when a remote buddy logs in/out, however
   current information makes the "in/out" determination unreliable.  Bugs are
   being filed against Gaim.  Therefore, this perk does not currently register
@@ -610,8 +635,8 @@ class HandleTableChange(Task.TableTask):
     @type first_child_por: L{POR}
     @param last_child_por: The L{POR} of last inserted row
     @type last_child_por: L{POR}
-    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
-      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @param layer: Layer on which the event occurred, one of L{Task.LAYER_FOCUS},
+      L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
     @type layer: integer
     '''
     #print "GaimPerk::executeTableRowInserted()", por, first_child_por, \
@@ -643,8 +668,8 @@ class HandleTableChange(Task.TableTask):
     @type first_child_por: L{POR}
     @param last_child_por: The L{POR} of last deleted row
     @type last_child_por: L{POR}
-    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
-      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @param layer: Layer on which the event occurred, one of L{Task.LAYER_FOCUS},
+      L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
     @type layer: integer
     '''
     #print "GaimPerk::executeTableRowDeleted()" , por, first_child_por, \
